@@ -302,6 +302,10 @@ class PersistentMap(MutableMapping):
             self._compress = lambda data: data
             self._decompress = lambda data: data
 
+        # if this pmap is an index, the table-pmap the index-pmap is attached to
+        self._index_attached_to = None
+
+        # if this pmap is NOT an index, any indexes attached to this (table-)pmap
         self._indexes = {}
 
     def indexes(self):
@@ -311,6 +315,14 @@ class PersistentMap(MutableMapping):
         """
         return sorted(self._indexes.keys())
 
+    def is_index(self):
+        """
+        Flag indicating whether this pmap is used as an index.
+
+        :return:
+        """
+        return self._index_attached_to is not None
+
     def attach_index(self, name, pmap, fkey, nullable=False, unique=True):
         """
 
@@ -327,9 +339,16 @@ class PersistentMap(MutableMapping):
         assert type(nullable) == bool
         assert type(unique) == bool
 
+        if self._index_attached_to:
+            raise Exception('cannot attach an index to an index (this pmap is already an index attached to {})'.format(
+                self._index_attached_to))
+        if pmap._index_attached_to:
+            raise Exception('index already attached (to {})'.format(pmap._index_attached_to))
         if name in self._indexes:
             raise Exception('index with name "{}" already exists'.format(name))
+
         self._indexes[name] = Index(name, fkey, pmap, nullable, unique)
+        pmap._index_attached_to = self
 
     def detach_index(self, name):
         """
@@ -707,6 +726,14 @@ class MapUuidUuidUuid(_types._UuidUuidKeysMixin, _types._UuidValuesMixin, Persis
         PersistentMap.__init__(self, slot=slot, compress=compress)
 
 
+class MapUuidTimestampUuid(_types._UuidTimestampKeysMixin, _types._UuidValuesMixin, PersistentMap):
+    """
+    Persistent map with (UUID, timestamp) keys and UUID values.
+    """
+    def __init__(self, slot=None, compress=None):
+        PersistentMap.__init__(self, slot=slot, compress=compress)
+
+
 class MapUuidStringUuid(_types._UuidStringKeysMixin, _types._UuidValuesMixin, PersistentMap):
     """
     Persistent map with (UUID, string) keys and UUID values.
@@ -939,6 +966,22 @@ class MapStringUuid(_types._StringKeysMixin, _types._UuidValuesMixin, Persistent
         PersistentMap.__init__(self, slot=slot, compress=compress)
 
 
+class MapStringStringUuid(_types._StringStringKeysMixin, _types._UuidValuesMixin, PersistentMap):
+    """
+    Persistent map with (string, string) keys and UUID (16 bytes) values.
+    """
+    def __init__(self, slot=None, compress=None):
+        PersistentMap.__init__(self, slot=slot, compress=compress)
+
+
+class MapStringStringStringUuid(_types._StringStringStringKeysMixin, _types._UuidValuesMixin, PersistentMap):
+    """
+    Persistent map with (string, string, string) keys and UUID (16 bytes) values.
+    """
+    def __init__(self, slot=None, compress=None):
+        PersistentMap.__init__(self, slot=slot, compress=compress)
+
+
 class MapStringJson(_types._StringKeysMixin, _types._JsonValuesMixin, PersistentMap):
     """
     Persistent map with string (utf8) keys and JSON values.
